rm(list=ls())

# required packages

library(timeSeries)
library(fpp)
library(rugarch)
library(ks)

setwd('c:\\users\\marco.bee\\dropbox\\didattica\\2016_2017\\Reaction\\data\\')

rawdata <- readSeries('IBM.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
IBMlr <- diff(log(rawdata),trim=TRUE)

# (a) )fit ARIMA-GARCH on log-returns

n <- length(IBMlr)
par(mfrow=c(3,1))
plot(IBMlr,main='Log-returns')
Acf(IBMlr,main='Log-returns')
Acf(IBMlr,main='Log-returns',type='partial')
par(mfrow=c(1,1))
hist(IBMlr,prob=T,ylim=c(0,40),main='Histogram of log-returns')
# h = hscv(IBMlr,binned=F)
fhat <- kde(as.vector(IBMlr),h=0.007) # .011
plot(fhat,add=T)
x <- seq(min(IBMlr),max(IBMlr),length.out=1000)
lines(x,dnorm(x,mean(IBMlr),sd(IBMlr)),col='red')
leg.txt <- c("Kernel density","Normal")
legend("topleft", leg.txt, lty=c(1,1),col=c('black','red'))
sk <- skewness(IBMlr)
ku <- kurtosis(IBMlr,method='moment')
print(c(sk,ku))

p <- 1
q <- 1
spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
fitarmagarch1111 <- ugarchfit(data=IBMlr,spec=spec)
plot(fitarmagarch1111)
resarmagarch1111 <- residuals(fitarmagarch1111)/sigma(fitarmagarch1111)
par(mfrow=c(2,1))
plot(as.timeSeries(resarmagarch1111),main='Residuals - GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - GARCH(1,1) on log-returns')
Pars <- coef(fitarmagarch1111)
ARMApars <- Pars[1:(p+q+1)]
GARCHpars <- Pars[(p+q+2):length(Pars)]
ucvar <- GARCHpars[1]/(1-GARCHpars[2]-GARCHpars[3])
print(c(as.vector(ucvar),var(IBMlr)))
uckurt <- 3*(1-(GARCHpars[2]+GARCHpars[3])^2)/(1-(GARCHpars[2]+GARCHpars[3])^2-2*GARCHpars[2]^2)
print(c(as.vector(uckurt),ku))
resarmagarch1111 <- residuals(fitarmagarch1111)/sigma(fitarmagarch1111)
par(mfrow=c(2,1))
plot(as.timeSeries(resarmagarch1111),main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
par(mfrow=c(1,1))

# forecasting

fore <- ugarchforecast(fitarmagarch1111, n.ahead = 100, n.roll = 0, out.sample = 0)
plot(fore)

# (b) )GARCH-M

spec = ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE, archm=T, archpow=1),distribution.model="norm")
fitGARCHM <- ugarchfit(data=IBMlr,spec=spec)
Pars <- coef(fitGARCHM)
ARMAparsM <- Pars[1:(p+q+2)]
resGARCHM <- residuals(fitGARCHM)/sigma(fitGARCHM)

# (c) ARIMA-GARCH with Student-t(6) innovations

p <- 1
q <- 1
spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="std",fixed.pars = list(shape=6))
fitarmagarcht6 <- ugarchfit(data=IBMlr,spec=spec)
plot(fitarmagarcht6)
resarmagarcht6 <- residuals(fitarmagarcht6)/sigma(fitarmagarcht6)

# (d) ARIMA-GARCH with Student-t innovations

p <- 1
q <- 1
spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="std")
fitarmagarcht <- ugarchfit(data=IBMlr,spec=spec)
plot(fitarmagarcht)
resarmagarcht <- residuals(fitarmagarcht)/sigma(fitarmagarcht)
Pars <- coef(fitarmagarcht)
covmat <- vcov(fitarmagarcht)
stdev <- sqrt(diag(covmat))
Z = (Pars[length(Pars)] - 6) / stdev[length(stdev)]
pval = 2*pnorm(Z)

# (e) EGARCH

p <- 1
q <- 1
spec = ugarchspec(variance.model=list(model="eGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
fitEGARCH <- ugarchfit(data=IBMlr,spec=spec)
Pars <- coef(fitEGARCH)
ARMAparsE <- Pars[1:(p+q+1)]
resEGARCH <- residuals(fitEGARCH)/sigma(fitEGARCH)

# (f) comparison

infocriteria(fitarmagarch1111)
infocriteria(fitGARCHM)
infocriteria(fitarmagarcht6)
infocriteria(fitarmagarcht)
infocriteria(fitEGARCH)

par(mfrow=c(3,2))
plot(sigma(fitarmagarch1111))
plot(sigma(fitGARCHM))
plot(sigma(fitarmagarcht6))
plot(sigma(fitarmagarcht))
plot(sigma(fitEGARCH))

