rm(list=ls())

# required packages

library(timeSeries)
library(fpp)
library(rugarch)
library(knitr)

setwd('c:\\users\\marco.bee\\dropbox\\didattica\\2016_2017\\Reaction\\data\\')
GSPC <- readSeries('GSPC.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
GSPClr <- diff(log(GSPC),trim=TRUE)
n <- length(GSPClr)

# Integrated GARCH

p <- 1
q <- 1
spec = ugarchspec(variance.model=list(model="iGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm", fixed.pars=list(omega=0))
fitIGARCH <- ugarchfit(data=GSPClr,spec=spec)
Pars <- coef(fitIGARCH)
ARMAparsI <- Pars[1:(p+q+1)]
resIGARCH <- residuals(fitIGARCH)/sigma(fitIGARCH)

# Exponential GARCH

spec = ugarchspec(variance.model=list(model="eGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
fitEGARCH <- ugarchfit(data=GSPClr,spec=spec)
Pars <- coef(fitEGARCH)
ARMAparsE <- Pars[1:(p+q+1)]
resEGARCH <- residuals(fitEGARCH)/sigma(fitEGARCH)

# GARCH-M

spec = ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE, archm=T, archpow=1),distribution.model="norm")
fitGARCHM <- ugarchfit(data=GSPClr,spec=spec)
Pars <- coef(fitGARCHM)
ARMAparsM <- Pars[1:(p+q+2)]
resGARCHM <- residuals(fitGARCHM)/sigma(fitGARCHM)

# compare

par(mfrow=c(3,1))
Acf(resIGARCH)
Acf(resEGARCH)
Acf(resGARCHM)

# use information criteria

models = 1:3
model.list = list()
for (p in models)
{
  garchfit = ugarchspec(variance.model = list(model = c("sGARCH","iGARCH","eGARCH")[p], garchOrder = c(1, 1)), mean.model = list(armaOrder = c(1, 1), include.mean = TRUE), distribution.model = c("norm", "snorm", "std", "sstd", "ged", "sged", "nig", "ghyp", "jsu")[1])
  garchfit1 = ugarchfit(spec = garchfit,data=GSPClr)
  model.list[[p]] = garchfit1
}
names(model.list) <- c("sGARCH","iGARCH","eGARCH")
fit.mat = sapply(model.list, infocriteria)
rownames(fit.mat) = rownames(infocriteria(model.list[[1]]))
kable(fit.mat)

# forecasting

foreIGARCH <- ugarchforecast(fitIGARCH, n.ahead = 100, n.roll = 0, out.sample = 0)
plot(foreIGARCH)
foreEGARCH <- ugarchforecast(fitEGARCH, n.ahead = 100, n.roll = 0, out.sample = 0)
plot(foreEGARCH)
foreGARCHM <- ugarchforecast(fitGARCHM, n.ahead = 100, n.roll = 0, out.sample = 0)
plot(foreGARCHM)

# IGARCH VaR computation

p <- 0.01
res <- residuals(fitIGARCH)
vola <- sigma(fitIGARCH)
mu1 <- ARMAparsI[1] 
VaRIGARCH <- mu1 + qnorm(p) * foreIGARCH@forecast$sigmaFor[1]

# EGARCH VaR computation

p <- 0.01
res <- residuals(fitEGARCH)
vola <- sigma(fitEGARCH)
mu1 <- ARMAparsE[1]
VaREGARCH <- mu1 + qnorm(p) * foreEGARCH@forecast$sigmaFor[1]

# GARCH-M VaR computation

p <- 0.01
res <- residuals(fitGARCHM)
vola <- sigma(fitGARCHM)
mu1 <- ARMAparsM[1] + ARMAparsM[2] * foreGARCHM@forecast$sigmaFor[1]^2
VaRMGARCH <- mu1 + qnorm(p) * foreGARCHM@forecast$sigmaFor[1]


