rm(list=ls())

# required packages

library(timeSeries)
library(fpp)
library(rugarch)
library(ks)

setwd('c:\\users\\marco.bee\\dropbox\\didattica\\2016_2017\\Reaction\\data\\')

GSPC <- readSeries('GSPC.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
GSPCd1 <- diff(GSPC,trim=TRUE)

# fit GARCH

spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(0,0), include.mean=TRUE),distribution.model="norm") # std
fitgarch11 <- ugarchfit(data=GSPCd1,spec=spec)
plot(fitgarch11)
resgarch11 <- residuals(fitgarch11)/sigma(fitgarch11) # alternative: fitgarch11@fit$residuals/fitgarch11@fit$sigma
par(mfrow=c(2,1))
plot(as.timeSeries(resgarch11),main='Residuals - GARCH(1,1) on first differences')
Acf(resgarch11,main='Residuals - GARCH(1,1) on first differences') # same result obtained by selecting 10 in command plot(garch.filter11)

# use a prespecified GARCH model to filter data (no fitting)

spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(0,0), include.mean=TRUE),distribution.model="norm", fixed.pars = as.list(coef(fitgarch11)))
garch.filter11 <- ugarchfilter(data = GSPCd1, spec = spec)

# fit ARIMA-GARCH on log-returns

GSPClr <- diff(log(GSPC),trim=TRUE)
n <- length(GSPClr)
par(mfrow=c(3,1))
plot(GSPClr,main='Log-returns')
Acf(GSPClr,main='Log-returns')
Acf(GSPClr,main='Log-returns',type='partial')
par(mfrow=c(1,1))
hist(GSPClr,prob=T,ylim=c(0,40),main='Histogram of log-returns')
# h = hscv(GSPClr,binned=F)
fhat <- kde(as.vector(GSPClr),h=0.007) # .011
plot(fhat,add=T)
x <- seq(min(GSPClr),max(GSPClr),length.out=1000)
lines(x,dnorm(x,mean(GSPClr),sd(GSPClr)),col='red')
leg.txt <- c("Kernel density","Normal")
legend("topleft", leg.txt, lty=c(1,1),col=c('black','red'))
sk <- skewness(GSPClr)
ku <- kurtosis(GSPClr,method='moment')
print(c(sk,ku))

p <- 1
q <- 1
spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
fitarmagarch1111 <- ugarchfit(data=GSPClr,spec=spec)
plot(fitarmagarch1111)
resarmagarch1111 <- residuals(fitarmagarch1111)/sigma(fitarmagarch1111)
par(mfrow=c(2,1))
plot(as.timeSeries(resarmagarch1111),main='Residuals - GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - GARCH(1,1) on log-returns')
Pars <- coef(fitarmagarch1111)
ARMApars <- Pars[1:(p+q+1)]
GARCHpars <- Pars[(p+q+2):length(Pars)]
ucvar <- GARCHpars[1]/(1-GARCHpars[2]-GARCHpars[3])
print(c(as.vector(ucvar),var(GSPClr)))
uckurt <- 3*(1-(GARCHpars[2]+GARCHpars[3])^2)/(1-(GARCHpars[2]+GARCHpars[3])^2-2*GARCHpars[2]^2)
print(c(as.vector(uckurt),ku))
resarmagarch1111 <- residuals(fitarmagarch1111)/sigma(fitarmagarch1111)
par(mfrow=c(2,1))
plot(as.timeSeries(resarmagarch1111),main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
par(mfrow=c(1,1))

# forecasting

fore <- ugarchforecast(fitarmagarch1111, n.ahead = 100, n.roll = 0, out.sample = 0)
plot(fore)

# VaR computation

pp <- 0.01
res <- residuals(fitarmagarch1111)
vola <- sigma(fitarmagarch1111)
mu1 <- ARMApars[1] + ARMApars[2]* GSPClr[n] -  ARMApars[3]* vola[n] * res[n] # Tsay p. 263
VaRGARCH <- mu1 + qnorm(pp) * fore@forecast$sigmaFor[1]

####################################################################
# USD/EUR exchange rate (quantity of USD for 1 EUR)
####################################################################

USDEUR <- readSeries('USDEUR.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
par(mfrow=c(2,1))
plot(USDEUR,main='Levels')
Acf(USDEUR,main='Levels')

adf.res <- adf.test(USDEUR)
kpss.res <- kpss.test(USDEUR)

# log-returns

USDEURlr <- diff(log(USDEUR),trim=TRUE)
n <- length(USDEURlr)
par(mfrow=c(3,1))
plot(USDEURlr,main='Log-returns')
Acf(USDEURlr,main='Log-returns')
Acf(USDEURlr,main='Log-returns',type='partial')
par(mfrow=c(1,1))
hist(USDEURlr,prob=T,ylim=c(0,70),main='Histogram of log-returns')
# h = hscv(USDEURlr,binned=F)
fhat <- kde(as.vector(USDEURlr),h=0.003) # .011
plot(fhat,add=T)
x <- seq(min(USDEURlr),max(USDEURlr),length.out=1000)
lines(x,dnorm(x,mean(USDEURlr),sd(USDEURlr)),col='red')
leg.txt <- c("Kernel density","Normal")
legend("topleft", leg.txt, lty=c(1,1),col=c('black','red'))
sk <- skewness(USDEURlr)
ku <- kurtosis(USDEURlr,method='moment')
print(c(sk,ku))

# fit ARMA-GARCH

spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
fitarmagarch1111 <- ugarchfit(data=USDEURlr,spec=spec)
plot(fitarmagarch1111)
resarmagarch1111 <- residuals(fitarmagarch1111)/sigma(fitarmagarch1111)
par(mfrow=c(2,1))
plot(as.timeSeries(resarmagarch1111),main='Residuals - GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - GARCH(1,1) on log-returns')
Pars <- coef(fitarmagarch1111)
ARMApars <- Pars[1:(p+q+1)]
GARCHpars <- Pars[(p+q+2):length(Pars)]
ucvar <- GARCHpars[1]/(1-GARCHpars[2]-GARCHpars[3])
print(c(as.vector(ucvar),var(GSPClr)))
uckurt <- 3*(1-(GARCHpars[2]+GARCHpars[3])^2)/(1-(GARCHpars[2]+GARCHpars[3])^2-2*GARCHpars[2]^2)
print(c(as.vector(uckurt),ku))
resarmagarch1111 <- residuals(fitarmagarch1111)/sigma(fitarmagarch1111)
par(mfrow=c(2,1))
plot(as.timeSeries(resarmagarch1111),main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
par(mfrow=c(1,1))

# forecasting

fore <- ugarchforecast(fitarmagarch1111, n.ahead = 100, n.roll = 0, out.sample = 0)
plot(fore)

# VaR computation

pp <- 0.01
res <- residuals(fitarmagarch1111)
vola <- sigma(fitarmagarch1111)
mu1 <- ARMApars[1] + ARMApars[2]* USDEURlr[n] -  ARMApars[3]* vola[n] * res[n] # Tsay p. 263
VaRGARCH <- mu1 + qnorm(pp) * fore@forecast$sigmaFor[1] # Tsay p. 263
