rm(list=ls())

# required packages

library(timeSeries)
library(fpp)
library(fGarch)
library(ks)

setwd('c:\\users\\marco.bee\\dropbox\\didattica\\2016_2017\\Reaction\\data\\')

GSPC <- readSeries('GSPC.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
GSPCd1 <- diff(GSPC,trim=TRUE)

# fit GARCH

fitgarch11 <- garchFit(~garch(1,1),data=GSPCd1)
resgarch11 <- fitgarch11@residuals/fitgarch11@sigma.t # standardized residuals
par(mfrow=c(2,1))
plot(resgarch11,main='Residuals - GARCH(1,1) on first differences')
Acf(resgarch11,main='Residuals - GARCH(1,1) on first differences')
par(ask=TRUE)

# fit ARIMA-GARCH on log-returns

GSPClr <- diff(log(GSPC),trim=TRUE)
par(mfrow=c(3,1))
plot(GSPClr,main='Log-returns')
Acf(GSPClr,main='Log-returns')
Acf(GSPClr,main='Log-returns',type='partial')
par(mfrow=c(1,1))
hist(GSPClr,prob=T,ylim=c(0,40),main='Histogram of log-returns')
# h = hscv(GSPClr,binned=F)
fhat <- kde(as.vector(GSPClr),h=0.007) # .011
plot(fhat,add=T)
x <- seq(min(GSPClr),max(GSPClr),length.out=1000)
lines(x,dnorm(x,mean(GSPClr),sd(GSPClr)),col='red')
leg.txt <- c("Kernel density","Normal")
legend("topleft", leg.txt, lty=c(1,1),col=c('black','red'))
sk <- skewness(GSPClr)
ku <- kurtosis(GSPClr,method='moment')
print(c(sk,ku))

fitarmagarch1111 <- garchFit(~arma(1,1)+garch(1,1),data=GSPClr,include.mean=T,cond.dist='norm') # check the fGarch help for the conditional distributions
summary(fitarmagarch1111)
GARCHfit <- fitarmagarch1111@fit
AllPars <- GARCHfit$params$params
ARMApars <- AllPars[1:3]
GARCHpars <- AllPars[c(4,5,7)]
ucvar <- GARCHpars[1]/(1-GARCHpars[2]-GARCHpars[3])
print(c(as.vector(ucvar),var(GSPClr)))
uckurt <- 3*(1-(GARCHpars[2]+GARCHpars[3])^2)/(1-(GARCHpars[2]+GARCHpars[3])^2-2*GARCHpars[2]^2)
print(c(as.vector(uckurt),ku))
resarmagarch1111 <- fitarmagarch1111@residuals/fitarmagarch1111@sigma.t
par(mfrow=c(2,1))
plot(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
par(mfrow=c(1,1))
fore <- predict(fitarmagarch1111, n.ahead = 100, plot=T) # forget about meanError; standard deviation (Tsay, p. 95) )converges to the unconditional std

# USD/EUR exchange rate (quantity of USD for 1 EUR)

USDEUR <- readSeries('USDEUR.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
par(mfrow=c(2,1))
plot(USDEUR,main='Levels')
Acf(USDEUR,main='Levels')

adf.res <- adf.test(USDEUR)
kpss.res <- kpss.test(USDEUR)

# log-returns

USDEURlr <- diff(log(USDEUR),trim=TRUE)
par(mfrow=c(3,1))
plot(USDEURlr,main='Log-returns')
Acf(USDEURlr,main='Log-returns')
Acf(USDEURlr,main='Log-returns',type='partial')
par(mfrow=c(1,1))
hist(USDEURlr,prob=T,ylim=c(0,70),main='Histogram of log-returns')
# h = hscv(USDEURlr,binned=F)
fhat <- kde(as.vector(USDEURlr),h=0.003) # .011
plot(fhat,add=T)
x <- seq(min(USDEURlr),max(USDEURlr),length.out=1000)
lines(x,dnorm(x,mean(USDEURlr),sd(USDEURlr)),col='red')
leg.txt <- c("Kernel density","Normal")
legend("topleft", leg.txt, lty=c(1,1),col=c('black','red'))
sk <- skewness(USDEURlr)
ku <- kurtosis(USDEURlr,method='moment')
print(c(sk,ku))

# fit ARMA-GARCH

fitarmagarch1111 <- garchFit(~arma(1,1)+garch(1,1),data=USDEURlr,include.mean=T,cond.dist='norm') # check the fGarch help for the conditional distributions
summary(fitarmagarch1111)
GARCHfit <- fitarmagarch1111@fit
AllPars <- GARCHfit$params$params
ARMApars <- AllPars[1:3]
GARCHpars <- AllPars[c(4,5,7)]
ucvar <- GARCHpars[1]/(1-GARCHpars[2]-GARCHpars[3])
print(c(as.vector(ucvar),var(GSPClr)))
uckurt <- 3*(1-(GARCHpars[2]+GARCHpars[3])^2)/(1-(GARCHpars[2]+GARCHpars[3])^2-2*GARCHpars[2]^2)
print(c(as.vector(uckurt),ku))
resarmagarch1111 <- fitarmagarch1111@residuals/fitarmagarch1111@sigma.t
par(mfrow=c(2,1))
plot(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
Acf(resarmagarch1111,main='Residuals - ARMA(1,1)-GARCH(1,1) on log-returns')
par(mfrow=c(1,1))
fore <- predict(fitarmagarch1111, n.ahead = 100, plot=T) # forget about meanError; standard deviation (Tsay, p. 95) )converges to the unconditional std
