# Some simulated processes

rm(list=ls())
library(rugarch)

n = 100
set.seed(10)
y = rnorm(n)
par(mfrow=c(2,1))
plot(y,type="b",main='White noise')
Acf(y,main='White noise')
par(ask=T)
y1 <- sample(y,n) # reshuflle data
plot(y1,type="b",main='White noise')
Acf(y1,main='White noise')

# Change in variance (1)

y1 = rnorm(n/2)
y2 = rnorm(n/2,0,3)
y = c(y1,y2)
plot(y,type="b",main='Change in variance')

# Change in variance (2)

cc = seq(1,6,length.out = 100)
y = rnorm(n) * cc
plot(y,type="b",main='Change in variance')

# random walk

nobs <- 100
y1 <- c(0,cumsum(rnorm(nobs))) # use definition
plot(y1,type='b',main='Random walk')
y2 <- arima.sim(n = nobs, list(order=c(0,1,0))) # use R function arima.sim
plot(y2,type='b',main='Random walk')

# AR

y <- arima.sim(n = nobs, list(order = c(1,0,0), ar = .8897), sd = sqrt(1))
plot(y,main='AR(1)')
Acf(y,main='AR(1)')

# ARMA

y <- arima.sim(n = 63, list(ar = c(0.8897, -0.4858), ma = c(-0.2279, 0.2488)), sd = sqrt(0.1796))
plot(y,main='ARMA(2,2)')
Acf(y,main='ARMA(2,2)')

# ARMA-GARCH

set.seed(5)
n <- 600
garch.spec = ugarchspec(variance.model = list(garchOrder=c(1,1)), mean.model = list(armaOrder=c(1,2)), fixed.pars=list(mu = 0, ar1= .5, ma1 = .3, ma2 = -.3, omega=0.25, alpha1=0.05, beta1=0.3, beta2=-0.3))
garch.sim <- ugarchpath(garch.spec, n.sim=n)
x <- drop(garch.sim@path$seriesSim)
tt <- 1:length(x)
plot(tt, x, type="b",main='ARMA(1,2)-GARCH(1,1)')
plot(tt,sigma(garch.sim), type="b",main='ARMA(1,2)-GARCH(1,1) - Conditional volatility')

# double-check

spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(1,2), include.mean=TRUE), distribution.model="norm")
fitgarch <- ugarchfit(data=x,spec=spec)
