rm(list=ls())

library(timeSeries)
library(rugarch)

# data

setwd('c:\\users\\marco.bee\\dropbox\\didattica\\2016_2017\\Reaction\\data\\')
GSPC <- readSeries('GSPC.csv', header = TRUE, sep = ",", format="%d/%m/%Y")
pp <- .1 # VaR level
L <- 200 # window length
train <- 2450 # 2450
sd <- "2007-08-10"
ed <- "2017-08-09"
p <- 0
q <- 0
r_orig <- diff(log(GSPC),trim=T)
n_orig <- length(r_orig)
VaRsGARCH <- rep(0,n_orig-train)
VaRIGARCH <- rep(0,n_orig-train)
VaREGARCH <- rep(0,n_orig-train)
VaRMGARCH <- rep(0,n_orig-train)
for (i in (train+1):n_orig)
{
  r <- r_orig[(i-L+1):i]
  n <- length(r)
  p <- 1
  q <- 0
  spec <- ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm") # distribution.model="norm" or "std""
  fitgarch11 <- ugarchfit(data=r,spec=spec)
  Pars <- coef(fitgarch11)
  ARMApars <- Pars[1:(p+q+1)]
  vola <- sigma(fitgarch11)
  res <- residuals(fitgarch11)
  fore <- ugarchforecast(fitgarch11, n.ahead = 1, n.roll = 0, out.sample = 0)
  mu1 <- as.numeric(ARMApars[1] + ARMApars[2]* r[n] -  ARMApars[3]* vola[n] * res[n])

# uncomment line 41 if distribution.model="norm" at line 29, uncomment lines 42-43 if distribution.model="std" at line 29

  VaRsGARCH[i-train] <- mu1 + qnorm(pp) * fore@forecast$sigmaFor[1] # normal VaR
#  dgf <- Pars[length(Pars)]
#  VaRsGARCH[i-train] <- mu1 + (qt(pp,dgf)/sqrt(dgf/(dgf-2))) * fore@forecast$sigmaFor[1] # standardized t VaR

# IGARCH

  spec <- ugarchspec(variance.model=list(model="iGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
  fitgarch11 <- ugarchfit(data=r,spec=spec)
  Pars <- coef(fitgarch11)
  ARMApars <- Pars[1:(p+q+1)]
  vola <- sigma(fitgarch11)
  res <- residuals(fitgarch11)
  fore <- ugarchforecast(fitgarch11, n.ahead = 1, n.roll = 0, out.sample = 0)
  mu1 <- as.numeric(ARMApars[1] + ARMApars[2]* r[n] -  ARMApars[3]* vola[n] * res[n])
  VaRIGARCH[i-train] <- mu1 + qnorm(pp) * fore@forecast$sigmaFor[1]

# EGARCH

  spec <- ugarchspec(variance.model=list(model="eGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE),distribution.model="norm")
  fitgarch11 <- ugarchfit(data=r,spec=spec)
  Pars <- coef(fitgarch11)
  ARMApars <- Pars[1:(p+q+1)]
  vola <- sigma(fitgarch11)
  res <- residuals(fitgarch11)
  fore <- ugarchforecast(fitgarch11, n.ahead = 1, n.roll = 0, out.sample = 0)
  mu1 <- as.numeric(ARMApars[1] + ARMApars[2]* r[n] -  ARMApars[3]* vola[n] * res[n])
  VaREGARCH[i-train] <- mu1 + qnorm(pp) * fore@forecast$sigmaFor[1]

# GARCH-M

  p <- 0
  q <- 0
  spec = ugarchspec(variance.model=list(model="sGARCH", garchOrder=c(1,1)),mean.model=list(armaOrder=c(p,q), include.mean=TRUE, archm=T, archpow=1),distribution.model="norm")
  fitgarch11 <- ugarchfit(data=r,spec=spec)
  Pars <- coef(fitgarch11)
  ARMApars <- Pars[1:(p+q+2)]
  vola <- sigma(fitgarch11)
  res <- residuals(fitgarch11)
  fore <- ugarchforecast(fitgarch11, n.ahead = 1, n.roll = 0, out.sample = 0)
  mu1 <- as.numeric(ARMApars[1] + ARMApars[2] * fore@forecast$sigmaFor[1])
  VaRMGARCH[i-train] <- mu1 + qnorm(pp) * fore@forecast$sigmaFor[1]
}

#  # backtesting

BsGARCH <- VaRTest(pp, r_orig[(train+1):n_orig], VaRsGARCH)
BiGARCH <- VaRTest(pp, r_orig[(train+1):n_orig], VaRIGARCH)
BEGARCH <- VaRTest(pp, r_orig[(train+1):n_orig], VaREGARCH)
BMGARCH <- VaRTest(pp, r_orig[(train+1):n_orig], VaRMGARCH)
